import sys

operating_system = sys.platform

if(operating_system != 'win32'):
    root_dir = '/tmp/.legistream/'
    ffmpeg_bin = 'ffmpeg'
else:
    root_dir = 'C:/temp/.legistream/'
    ffmpeg_bin = 'ffmpeg.exe'


import json
import random
from websocket import create_connection

lower_stream_id = '23181491'
upper_stream_id = '23195631'

stream_base = 'https://uhsakamai-a.akamaihd.net/syd01/'

rnd = random.randrange

def num_to_hex(n):
    return hex(n)[2:]

class Stream(object):
    @property
    def lower_is_live(self):
        return(self.__get_stream_status(lower_stream_id))

    @property
    def upper_is_live(self):
        return(self.__get_stream_status(upper_stream_id))

    @property
    def lower_stream_url(self):
        if(self.lower_is_live):
            try:
                return(stream_base + self.__get_stream_json(lower_stream_id)['streamFormats']['mp4/segmented']['contentAccess']['accessList'][0]['data']['path'])
            except:
                return ''
        else:
            return ''

    @property
    def upper_stream_url(self):
        if(self.upper_is_live):
            try:
                return(stream_base + self.__get_stream_json(upper_stream_id)['streamFormats']['mp4/segmented']['contentAccess']['accessList'][0]['data']['path'])
            except:
                return ''
        else:
            return ''

    @property
    def stream_urls(self):
        return({'lower': self.lower_stream_url, 'upper': self.upper_stream_url})

    def __get_stream_status(self, stream_id):
        try:
            return(bool(self.__get_stream_json(stream_id)['contentAvailable']))
        except:
            return(False)
        

    def __get_stream_json(self, stream_id):
        rnd = random.randrange
        if(stream_id == lower_stream_id):
            _id = 'https://www.parliament.sa.gov.au/about-parliament/broadcast-pages/lc'
        else:
            _id = 'https://www.parliament.sa.gov.au/'
        ws = create_connection('wss://r%d-1-%s-channel-wss-omega.ums.ustream.tv/1/ustream' % (rnd(1e8), stream_id), header=['User-Agent: Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.183 Safari/537.36'], origin='https://www.ustream.tv')
        args = {
            'type': 'viewer',
            'appId': 3,
            'appVersion': 2,
            'application': 'channel',
            'buildNumber': "2.26.7",
            'clusterHost': "r%d-1-%s-channel-omega.ums.services.video.ibm.com" % (rnd(1e8), stream_id),
            'rsid': '%s:%s' % (num_to_hex(rnd(1e8)), num_to_hex(rnd(1e8))),
            'rpin': '_rpin.%d' % rnd(1e15),
            'referrer': _id,
            'media': stream_id
        }
        ws.send(json.dumps({'cmd': 'connect', 'args': [args]}))
        stream_dat = json.loads(ws.recv())
        ws.close()
        try:
            return(stream_dat['args'][0]['stream'])
        except:
            raise Exception(args)


from PIL import Image
from requests import get
from bs4 import BeautifulSoup
from datetime import datetime
import os, m3u8, subprocess, imagehash, shutil
from . import common

filepath = os.path.dirname(os.path.realpath(__file__))

class Stream(object):
    lower_img_hash = 18446744069414592512
    upper_img_hash = 18446743326385257472
    
    @property
    def lower_stream_url(self):
        return BeautifulSoup(get('https://www.parliament.tas.gov.au/TBS/havideo.html').text, 'lxml').find('source')['src']

    @property
    def upper_stream_url(self):
        return BeautifulSoup(get('https://www.parliament.tas.gov.au/TBS/LCvideo.html').text, 'lxml').find('source')['src']

    @property
    def stream_urls(self):
        return({'lower': self.lower_stream_url, 'upper': self.upper_stream_url})


    @property
    def lower_is_live(self):
        try:
            if(abs(self.lower_img_hash - self.__get_vid_hash(self.lower_stream_url)) < 5):
                return(False)
            else:
                return(True)
        except:
            return False
        
    @property
    def upper_is_live(self):
        if(abs(self.upper_img_hash - self.__get_vid_hash(self.upper_stream_url)) < 5):
            return(False)
        else:
            return(True)

    def __cleanup(self, directory): # Clear Legistream's temp directory
        try:
            if(os.path.exists(directory)):
                shutil.rmtree(directory)
        except Exception as e:
            print(e)

    def __get_vid_hash(self, input_url):
        try:
            try:
                playlist_data = m3u8.loads(get(input_url).text)
            except:
                return None
            seg_uri = m3u8.loads(get(input_url.replace('playlist.m3u8', '') + playlist_data.data['playlists'][0]['uri']).text).data['segments'][-1]['uri']
            seg_ts = get(input_url.replace('playlist.m3u8', '') + seg_uri)
            current_time = str(datetime.now()).replace(':', '-').replace('.', '-').replace(' ', '_')
            if(not os.path.exists(common.root_dir)):
                os.mkdir(common.root_dir)

            seg_output_file = common.root_dir + current_time + '_tas_seg.ts'
            with open(seg_output_file, 'wb') as file:
                file.write(seg_ts.content)
            
            img_out = common.root_dir + current_time + '_tas_seg_out.jpg'
            
            command = ['-ss', '00:00:00', '-i', seg_output_file, '-frames:v', '1', img_out]
            subprocess.run([common.ffmpeg_bin] + command, capture_output=True)
            _hash = int(str(imagehash.average_hash((Image.open(img_out)))), 16)
            self.__cleanup(common.root_dir)
            return _hash
        except:
            raise Exception('Invalid input URL: ' + input_url)


import json, re
from requests import get
from bs4 import BeautifulSoup

refer_url = 'https://api-v3.switchmedia.asia/'
api_url = [refer_url + '277/assets/get-data?asset=11725', '&detail=verbose&format=json']
stream_base_url = 'https://dps-live-hls.global.ssl.fastly.net/hls/'
num_range = range(68, 71)
page_soup = BeautifulSoup(get('https://www.aph.gov.au/Watch_Read_Listen').text, 'lxml')

class Stream(object):
    def __init__(self):
        self.__get_stream_urls()
        self.__scrape_committees()

    def __scrape_committees(self):
        live_pages = []
        self.com_urls = []
        chk_list = BeautifulSoup(get('https://www.aph.gov.au/Watch_Read_Listen').text, 'lxml').find('section', {'id': 't1-content-panel'}).find_all('div', {'class': 'medium-7 columns'})
        for entry in chk_list:
            if(entry.text.strip()[-4:] == 'Live'):
                _stream_title = entry.text.strip()[:-4].strip()
                if(_stream_title == 'Senate' or _stream_title == 'House of Representatives' or _stream_title == 'Federation Chamber'):
                    pass
                else:
                    live_pages.append({'title': entry.text.strip()[:-4].strip(),'url': 'https://www.aph.gov.au' + entry.find('a')['onclick'][13:][:75]})
        if(not len(live_pages) == 0):
            for url in live_pages:
                try:
                    self.com_urls.append({'title': url['title'], 'url': json.loads(get('https://api-v3.switchmedia.asia/277/playback/getUniversalPlayerConfig?videoID=' + BeautifulSoup(get(url['url']).text, 'lxml').find('iframe')['src'][72:][:7] + '&playlistID=0&skinType=vcms&profile=regular&playerID=playerregular&format=json&bookmarkID=0&autoplay=true&referrer=https://www.aph.gov.au/News_and_Events/LiveMediaPlayer&siteID=277&cl=1').text)['media']['renditions'][0]['url']})
                except:
                    # This happens with audio-only streams, which Legistream doesn't support yet...
                    pass
    def __get_stream_urls(self):
        self.sittings = []
        self.sitting_div = page_soup.find('div', {'id': 'content'}).find_all('p', {'class': 'watch-live text-center'})
        for row in self.sitting_div:
            self.sittings.append(row.text.strip())
        for asset_id in num_range:
            json_data = json.loads(get(api_url[0] + str(asset_id) + api_url[1]).text)[0]
            prefix_id = json_data['image'].replace('https://downloads.switchmedia.asia/filestore/getimage.php?siteID=', '')[:3]
            # prefix_id = '277'
            stream_id = json_data['title'][-3:]
            house_id = json_data['synopsis'].replace('Live Stream: ', '')[:3]
            url_suffix = prefix_id + '_' + house_id + stream_id + '_18000.m3u8'
            url = stream_base_url + url_suffix
            if(house_id == 'HOR'):
                self.lower_stream_url = url
            elif(house_id == 'COM'):
                self.committee_stream_url = url
            elif(house_id == 'SEN'):
                self.upper_stream_url = url
            else:
                raise Exception('An error occurred, got house_id ' + str(house_id))

    @property
    def lower_is_live(self):
        if(self.sittings[0] == 'Not sitting'):
           return(False)
        else:
           return(True)
        
    @property
    def upper_is_live(self):
        if(self.sittings[2] == 'Not sitting'):
           return(False)
        else:
           return(True)
    
    @property
    def committee_is_live(self):
        if(self.sittings[1] == 'Not sitting' and not 'https://dps-live-hls.global.ssl.fastly.net/hls/277_COM109_18000.m3u8' in self.com_urls):
           return(False)
        else:
           return(True)
    
    @property
    def stream_urls(self):
        data_dict = {'lower': self.lower_stream_url, 'upper': self.upper_stream_url, 'committee': self.committee_stream_url, 'extra_committees': self.com_urls}
        for entry in data_dict:
            if(data_dict[entry] in data_dict['extra_committees']):
                data_dict['extra_committees'].remove(data_dict[entry])
        if(len(data_dict['extra_committees']) == 0):
            data_dict.pop('extra_committees', None)
        return data_dict


import m3u8, json, re
from requests import get
from bs4 import BeautifulSoup

CALENDAR_URL = 'http://tv.parliament.qld.gov.au/TV/PartialCalendar'

class Stream(object):
    @property
    def is_live(self):
        try:
            _m3u8 = get(self.stream_url)
            if(_m3u8.status_code != 200):
                return False

            self.m3u8_data = m3u8.parse(_m3u8).text
            if(self.m3u8_data['playlists']):
                return True
            else:
                return False
        except:
            return False

    @property
    def stream_url(self):
        _js = BeautifulSoup(get('http://tv.parliament.qld.gov.au/TV/PartialGetHomeContent').text, 'lxml').find('script').string.split('\n')
        
        for index,line in enumerate(_js):
            if(line.strip() == 'var videoSources = ['):
                try:
                    return json.loads(_js[index + 1].replace('];', '').replace(',', '').strip().replace("'", '"'))['file']
                except:
                    return ''

    @property
    def stream_title(self):
        if(self.is_live):
            soup = BeautifulSoup(get(CALENDAR_URL).text, 'lxml')
            try:
                _title = soup.find_all('h3')[-1].text.strip()
                if(_title == 'House Sitting Date'):
                    return 'Legislative Assembly'
                else:
                    return _title
            except:
                return ''
        else:
            return ''

    @property
    def stream_urls(self):
        if(self.is_live):
            return {self.stream_title: self.stream_url}
        else:
            return {}


import m3u8, json
from requests import get

api_url = 'http://aod.parliament.act.gov.au/api/video'

class Stream(object):
    __api_call = json.loads(get(api_url).text)['live']

    @property
    def lower_stream_url(self):
        return self.__api_call['url'] + self.__api_call['extension']
    
    @property
    def lower_is_live(self):
        if(m3u8.parse(get(self.lower_stream_url).text)['playlists']):
            return True
        else:
            return False
    
    @property
    def stream_urls(self):
        return {'lower': self.lower_stream_url}


import m3u8, json
from requests import get
from requests.sessions import SessionRedirectMixin

API_URL = 'https://www.parliament.vic.gov.au/video-streaming/v7/scripts/akamai_desktop.json?_=1612478772960'

class Stream(object):
    def __init__(self):
        self.__get_api_data(API_URL)

    def __get_api_data(self, url):
        self.api_data = json.loads(get(url).text)
    
    def lower_stream(self):
        return self.__get_stream('Assembly Live Broadcast - Desktop')

    def upper_stream(self):
        return self.__get_stream('Council Live Broadcast - Desktop')

    def comm_stream(self):
        return self.__get_stream('Committee Live Broadcast - Desktop')

    def __get_stream(self, title):
        for stream in self.api_data:
            if(stream['title'] == title):
                return stream

    @property
    def lower_stream_url(self):
        try:
            return f'https:{self.lower_stream()["checksrc"]}'.replace('400', '900')
        except:
            return None

    @property
    def lower_is_live(self):
        try:
            if(get(self.lower_stream_url).status_code != 200):
                return False
            else:
                return True
        except:
            return False

    @property
    def upper_stream_url(self):
        try:
            return f'https:{self.upper_stream()["checksrc"]}'.replace('400', '900')
        except:
            return None

    @property
    def upper_is_live(self):
        try:
            if(get(self.upper_stream_url).status_code != 200):
                return False
            else:
                return True
        except:
            return False

    @property
    def committee_stream_url(self):
        try:
            return f'https:{self.comm_stream()["checksrc"]}'.replace('400', '900')
        except:
            return None

    @property
    def committee_is_live(self):
        try:
            if(get(self.committee_stream_url).status_code != 200):
                return False
            else:
                return True
        except:
            return False